/*
 *  Copyright (c) 1995 John T. Kohl
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. The name of the author may not be used to endorse or promote products
 *     derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR `AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 * 
 */
#include "forms.h"
#include "battery.h"
#include <sys/types.h>
#include <sys/ioctl.h>
#include <machine/apmvar.h>
#include <fcntl.h>
#include <err.h>
#include <errno.h>

FD_batterylife *fd_batterylife;

int apmfd;
struct apm_power_info pi;

int
main(int argc, char *argv[])
{
    int val;

    apmfd = open("/dev/apm", O_RDONLY);
    if (apmfd == -1)
	err(1,"cannot open /dev/apm");
    if (ioctl(apmfd, APM_IOC_GETPOWER, &pi) == -1)
	err(1, "cannot fetch APM power info");

    fl_initialize(argv[0], "BatteryLife", 0, 0, &argc, argv);
    fd_batterylife = create_form_batterylife();

    fl_set_chart_bounds(fd_batterylife->battery_pie, 0.0, 110.0);
    fl_set_chart_maxnumb(fd_batterylife->battery_pie, 1);
    battery_timer_up(fd_batterylife->check_timer, 0);

    /* show the first form */
    fl_show_form(fd_batterylife->batterylife,FL_PLACE_ASPECT|FL_FREE_SIZE,
		 FL_FULLBORDER, "Battery Life");
    while (fl_do_forms())
	;

}

void
battery_timer_up(FL_OBJECT *timer, long val)
{
    char str[16];

    if (ioctl(apmfd, APM_IOC_GETPOWER, &pi) == -1) {
	fl_show_alert("Cannot fetch APM power info",
		      strerror(errno), 0, 1);
	errx(1, "cannot fetch APM power info");
    }
    sprintf(str,"%02d%%    ",pi.battery_life);
    fl_set_object_label(fd_batterylife->battery_pie,str);
    fl_clear_chart(fd_batterylife->battery_pie);
    if (pi.ac_state == APM_AC_ON || pi.battery_state == APM_BATT_CHARGING) {
	fl_add_chart_value(fd_batterylife->battery_pie,
			   (double) pi.battery_life, "charging", FL_BLUE);
    } else {
	if (pi.battery_life == APM_BATT_UNKNOWN)
	    fl_add_chart_value(fd_batterylife->battery_pie, 0,
			       "on battery", FL_GRAY63);
	else if (pi.battery_life > 45)
	    fl_add_chart_value(fd_batterylife->battery_pie,
			       (double) pi.battery_life, "on battery",
			       FL_GREEN);
	else if (pi.battery_life > 20)
	    fl_add_chart_value(fd_batterylife->battery_pie,
			       (double) pi.battery_life, "on battery",
			       FL_YELLOW);
	else
	    fl_add_chart_value(fd_batterylife->battery_pie,
			       (double) pi.battery_life, "on battery",
			       FL_RED);
    }
    fl_redraw_form(fd_batterylife->batterylife);
    fl_set_timer(timer, (double) 60.0);
}

void
battery_exit(FL_OBJECT *button, long val)
{
    exit(0);
}
